'use strict';
const Generator = require('yeoman-generator');
const prompts = require('./prompts');
const globalConstants = require('../../constants');
const errors = require('../../classes/errors');
const chalk = require('chalk');
const prettier = require('gulp-prettier');
const gulpIf = require('gulp-if');
const gulpMatch = require('gulp-match');

module.exports = class extends Generator {
  constructor(...args) {
    super(...args);
    this._declareVariables();
    this._configureOptions();
    this._configureFileProcessing();
  }

  _configureFileProcessing() {
    this.registerTransformStream(
      gulpIf((file) => {
        const excludedFiles = ['*/typings.d.ts', '*/test.ts'];
        const isTs = gulpMatch(file, '*.ts');
        const isExcluded = excludedFiles.some((fileName) =>
          gulpMatch(file, fileName),
        );

        return isTs && !isExcluded;
      }, prettier({ singleQuote: true })),
    );
  }

  prompting() {
    const configurationFileName =
      this.options.config ||
      globalConstants.parameters.CONFIG_FILE_DEFAULT_NAME;

    if (this.options['create-config']) {
      return;
    }

    if (this._configurationFileExist(configurationFileName)) {
      this.props = this._getConfigurationFile(configurationFileName);
      return;
    }

    if (this.options.subgenerator) {
      return;
    }

    return this.prompt(prompts).then((props) => {
      this.props = props;
    });
  }

  writing() {
    if (this.options['create-config']) {
      this._executeConfigFileGenerator();
      return;
    }

    this._executeSubgeneratorByCriteria();
  }

  _executeConfigFileGenerator() {
    this._executeSubgenerator({
      route: 'util/c6-config',
    });
  }

  /**
   * Executes subgenerator depending on the criteria, this could be
   * if generator found or not configuration file.
   */
  _executeSubgeneratorByCriteria() {
    if (this.configurationFileExists) {
      process.env.NO_WORKSPACE_VERIFICATION = true;
      this._executeConfigurationFile();
      return;
    }

    if (this.options.subgenerator) {
      this._executeSubgenerator({
        route: this.options.subgenerator,
      });
    } else {
      this._executeSubgenerator({
        generatorType: this.props.generatorType,
        generator: this.props.generator,
      });
    }
  }

  /**
   * Helper method to reduce execution script. It execute the
   * subgenerator depending on received parameters.
   * @param {Object} params
   */
  _executeSubgenerator(params) {
    params = params || {};
    const route = params.route
      ? `../${params.route}`
      : `../${params.generatorType}/${params.generator}`;
    try {
      const resolvedRoute = require.resolve(route);
      this.composeWith(resolvedRoute, params.config || {});
    } catch (error) {
      this._errorHandler(error, route);
    }
  }

  _errorHandler(error, route) {
    switch (error.name) {
      case errors.FolderNotExistError.name:
      case errors.NotAWorkspaceError.name:
        this._logError(error.message);
        break;
      default:
        this._logError(`Cannot find generator ${route.replace('../', '')}`);
    }
  }

  _logError(message) {
    this.log(chalk.red(message));
  }

  /**
   * This method should be executed when this.props
   * contains information from configuration file.
   */
  _executeConfigurationFile() {
    try {
      Object.keys(this.props).forEach((generatorType) => {
        const generators = this.props[generatorType];
        Object.keys(generators).forEach((generator) => {
          const generatorConfiguration = generators[generator];
          this._executeSubgenerator({
            generatorType: generatorType,
            generator: generator,
            config: { props: generatorConfiguration },
          });
        });
      });
    } catch (error) {
      this.log(chalk.red('Configuration file is not well written.'));
    }
  }

  /**
   * Method executed on contructor to declare class
   * variables.
   */
  _declareVariables() {
    this.configurationFileExists = false;
  }

  /**
   * Method executed in constructor to configurate the options
   * received by cli.
   */
  _configureOptions() {
    this.option('config', { type: String });
    this.option('subgenerator', { type: String });
    this.option('create-config', { type: Boolean, default: false });
  }

  /**
   * Returns true if the file with the given fileName
   * exist.
   * @param {String} fileName
   */
  _configurationFileExist(fileName) {
    this.configurationFileExists = this.fs.exists(
      this.destinationPath(fileName),
    );
    return this.configurationFileExists;
  }

  /**
   * Return JSON object from configuration file
   * with the given fileName.
   * @param {String} fileName
   */
  _getConfigurationFile(fileName) {
    return this.fs.readJSON(this.destinationPath(fileName));
  }
};
