'use strict';
const Generator = require('yeoman-generator');
const workspaceHelper = require('./workspace.helper');
const { scanPath } = require('folder-scanner');
const path = require('path');
const { pascalCase, camelCase } = require('change-case');
const ejs = require('ejs');
const gulpIf = require('gulp-if');
const gulpMatch = require('gulp-match');
const prettier = require('gulp-prettier');

const generatorsList = [
  'application/site',
  'application/enrollment',
  'module/participant',
  'module/site',
  'util/c6-config',
];

module.exports = {
  registrateGenerators(injectables) {
    const { env } = injectables;
    env.register(require.resolve('../generators/app'), 'c6');
    generatorsList.forEach((generator) => {
      env.register(require.resolve(`../generators/${generator}`), generator);
    });
  },

  moduleGeneratorFactory(configName) {
    return class extends Generator {
      constructor(...args) {
        super(...args);
        const destinationPath = this.destinationPath();
        workspaceHelper.verifyWorkspaceFolder(destinationPath);
        workspaceHelper.verifyConfigFolder(destinationPath);
        this._configureFileProcessing();
      }

      writing() {
        let scannedConfigurationFilesObjects;
        const destinationPath = this.destinationPath();
        const workspaceConfig = workspaceHelper.getWorkspaceConfig(
          destinationPath,
        );
        const moduleConfigFilesPath = path.resolve(
          destinationPath,
          workspaceConfig.configFilesPath,
          'module',
        );
        const clinical6ModuleListPath = path.resolve(
          destinationPath,
          workspaceConfig.clinical6ModuleListPath,
        );
        const configFileDestinyPath = this.destinationPath(
          moduleConfigFilesPath,
          `${configName}.module-config.ts`,
        );

        this._copyConfigurationTemplate(configFileDestinyPath);

        scannedConfigurationFilesObjects = this._getModuleConfigurationPaths(
          moduleConfigFilesPath,
        );

        this._pushNewConfigObjectName(
          configFileDestinyPath,
          scannedConfigurationFilesObjects,
        );

        this._copyConfigurationIndexFile(
          moduleConfigFilesPath,
          scannedConfigurationFilesObjects,
        );

        this._copyClinical6ModulesList(
          clinical6ModuleListPath,
          scannedConfigurationFilesObjects,
        );
      }

      _configureFileProcessing() {
        this.registerTransformStream(
          gulpIf((file) => {
            const excludedFiles = ['*/typings.d.ts', '*/test.ts'];
            const isTs = gulpMatch(file, '*.ts');
            const isExcluded = excludedFiles.some((fileName) =>
              gulpMatch(file, fileName),
            );

            return isTs && !isExcluded;
          }, prettier({ singleQuote: true })),
        );
      }

      _copyClinical6ModulesList(
        clinical6ModuleListPath,
        scannedConfigurationFilesObjects,
      ) {
        const template = this.fs.read(
          this.templatePath('clinical6-modules.ts.ejs'),
        );
        this.fs.write(
          clinical6ModuleListPath,
          ejs.render(template, {
            files: scannedConfigurationFilesObjects,
          }),
        );
      }

      _copyConfigurationIndexFile(
        moduleConfigFilesPath,
        scannedConfigurationFilesObjects,
      ) {
        const destinationIndexPath = path.resolve(
          moduleConfigFilesPath,
          'index.ts',
        );
        const template = this.fs.read(this.templatePath('index.ts.ejs'));
        this.fs.write(
          destinationIndexPath,
          ejs.render(template, {
            files: scannedConfigurationFilesObjects,
          }),
        );
      }

      _pushNewConfigObjectName(
        configFileDestinyPath,
        scannedConfigurationFilesObjects,
      ) {
        const configObject = this._getFileObjectByFileName(
          this._getFileNameByRoute(configFileDestinyPath),
        );

        Object.assign(configObject, {
          fileName: configObject.fileName.replace('.ts', ''),
        });

        if (
          !scannedConfigurationFilesObjects.some(
            (configObjectStored) =>
              configObjectStored.fileName === configObject.fileName,
          )
        ) {
          scannedConfigurationFilesObjects.push(configObject);
        }
      }

      _getModuleConfigurationPaths(moduleConfigFilesPath) {
        const response = scanPath(moduleConfigFilesPath)
          .filter((route) => !route.endsWith('index.ts'))
          .map(this._getFileNameByRoute)
          .map(this._getFileObjectByFileName)
          .map((fileObject) =>
            Object.assign(fileObject, {
              fileName: fileObject.fileName.replace('.ts', ''),
            }),
          );
        return response;
      }

      _getFileObjectByFileName(fileName) {
        const moduleConfigTypeString = '.module-config.ts';
        return {
          fileName,
          className: pascalCase(
            fileName.replace(moduleConfigTypeString, 'ModuleConfig'),
          ),
          moduleName: pascalCase(
            fileName.replace(moduleConfigTypeString, 'Module'),
          ),
          providerName: camelCase(fileName.replace('.ts', 'Provider')),
        };
      }

      _getFileNameByRoute(route) {
        let splittedRoute;
        if (process.platform === 'win32') {
          splittedRoute = route.split('\\');
        } else {
          splittedRoute = route.split('/');
        }
        return splittedRoute[splittedRoute.length - 1];
      }

      _copyConfigurationTemplate(configFileDestinyPath) {
        this.fs.copy(
          this.templatePath(`${configName}.module-config.ts.ejs`),
          configFileDestinyPath,
        );
      }
    };
  },
};
